local major = "EZLoatheb"
local minor = 1
local EZLoatheb = LibStub:NewLibrary(major, minor)

local mainTank
local healerData = {}
local healerRotationList = {}
local nonHealerList = {}
local playerShouldHeal = false
local eventHandlers = {}
local loatheb_hidden = false
local lastPingTime
local wasInParty = UnitInRaid("player") ~= nil

local healthBuffer = 600
local loathebMainFrame = CreateFrame("FRAME", "LoathebMainFrame", UIParent)
loathebMainFrame:SetPoint("CENTER", UIParent, "CENTER")
local COMM_PREFIX = "EZLoatheb"
C_ChatInfo.RegisterAddonMessagePrefix(COMM_PREFIX)

-- TODO:

local allTimers = {
    ["UPDATE_MED"] = 10,
    ["UPDATE_SEC"] = 1,
    ["UPDATE_FREQUENT"] = 0.2,
}

local corruptedMindID = {
    [29184] = true, [29195] = true, [29197] = true, [29199] = true
}

local function loathebHide()
    loatheb_hidden = true
    loathebMainFrame:Hide()
    for k, v in pairs(eventHandlers) do
        if k ~= "GROUP_ROSTER_UPDATE" then
            loathebMainFrame:UnregisterEvent(k); -- Register all events for which handlers have been defined
        end
    end
end

local function loathebShow()
    loatheb_hidden = false
    loathebMainFrame:Show()
    for k, v in pairs(eventHandlers) do
        loathebMainFrame:RegisterEvent(k); -- Register all events for which handlers have been defined
    end
end

SLASH_LOATHEB_SHOW1 = "/loatheb"
SlashCmdList["LOATHEB_SHOW"] = function(msg)
    if loatheb_hidden then
        loathebShow()
    else
        loathebHide()
    end
end

-- Cache commonly used functions
local UnitHealthMax = UnitHealthMax;
local UnitHealth = UnitHealth;
local InCombatLockdown = InCombatLockdown;
local UnitGUID = UnitGUID;

local needsUpdate = false
local mainTankHealthDiff = 0
local myHealAmount = 2300

local CTL = _G.ChatThrottleLib
local function sendAddonMessage(msg, target)
    if( target and strlen(msg) <= 240 ) then
        if CTL then
			CTL:SendAddonMessage("BULK", COMM_PREFIX, msg, "WHISPER", target)
		end
	end
end

-- Check Timer for Frequent Updates
local function checkResetTimer(timerName, nextTick, resetTime)
	if allTimers[timerName] > 0 then
		allTimers[timerName] = allTimers[timerName] - nextTick
		if allTimers[timerName] <= 0 then
			allTimers[timerName] = resetTime;
			return true;
		end
	end
	return false;
end

local function calculateHealAmount()
    local classFilename = UnitClassBase("player")
    local spellHeal = GetSpellBonusHealing()

    if classFilename == "PALADIN" then
        return math.floor(math.min(1700 + 0.7 * spellHeal, 3200))
    elseif classFilename == "PRIEST" then
        return math.floor(math.min(2100 + 0.85 * spellHeal, 3200))
    elseif classFilename == "DRUID" then
        return math.floor(math.min(2000 + spellHeal, 3200))
    elseif classFilename == "SHAMAN" then
        return math.floor(math.min(1700 + 0.85 * spellHeal, 3200))
    else
        return 0
    end
end

local function getMainTank()
    local partySize = GetNumGroupMembers()
    for i=1, partySize do
        local key = "raid" .. i
        if ((select(10, GetRaidRosterInfo(i))) == "MAINTANK") then
            mainTank = key
            -- TODO: is this the right place for this?
            --[[
            if loathebMainFrame.mainTankHealth then
                loathebMainFrame.mainTankHealth.text:SetText(mainTank)
                loathebMainFrame.mainTankHealth.healthBar:SetAttribute("unit", mainTank)
                loathebMainFrame.mainTankHealth.healthBar:SetAttribute("type", "target")
            end
            ]]
            return
        end
    end
    mainTank = nil
    --print("EZLoatheb: A single tank needs to be marked as Main Tank in /raidinfo")
end

local function pingAllHealers()
    lastPingTime = GetTime()
    for unitGUID, healerInfo in pairs(healerData) do
        if (healerInfo.lastPingTime and lastPingTime - healerInfo.lastPingTime > 5) or not healerInfo.lastPingTime then
            if UnitIsConnected(healerInfo.unitID) and UnitIsConnected(healerInfo.unitName) then
                sendAddonMessage("ping," .. myHealAmount, (UnitName(healerInfo.unitID)))
            end
        end
    end
end

-- Updates table healerData with UnitGUID to healer data of current party members
local function updateHealerList()
    myHealAmount = calculateHealAmount()
    -- healerData
    for unitGUID, val in pairs(healerData) do
        val.updated = false
    end

    for i=1, GetNumGroupMembers() do
        local key = "raid" .. i
        local classFilename = UnitClassBase(key)
        if classFilename == "PALADIN" or classFilename == "PRIEST" or classFilename == "DRUID" or classFilename == "SHAMAN" then
            if healerData[UnitGUID(key)] then
                healerData[UnitGUID(key)].updated = true
                healerData[UnitGUID(key)].unitID = key
            else
                healerData[UnitGUID(key)] = {
                    unitID = key, 
                    ezLoatheb = false, 
                    unitName = UnitName(key), 
                    active = true, 
                    ezLoatheb = UnitIsUnit(key, "player"),
                    healingDebuff = 0,
                    updated = true
                }
                if healerData[UnitGUID(key)].unitName == "Unknown" then
                    needsUpdate = true
                    healerData[UnitGUID(key)] = nil
                    return
                end
            end
        end
    end

    for unitGUID, val in pairs(healerData) do
        if not val.updated then
            healerData[unitGUID] = nil
        end
    end

    -- TODO: make sure this is needed
    pingAllHealers()
end

--NEW
-- update lists of healers with/without addon installed
local function updateHealerRotationList()
    healerRotationList = {}
    nonHealerList = {}

    local partySize = GetNumGroupMembers()
    for i=1, partySize do
        key = "raid" .. i
        if ((select(10, GetRaidRosterInfo(i))) == "MAINTANK") then
            mainTank = key
            break
        end
    end

    -- TODO: if custom order, else
    local partySize = GetNumGroupMembers()
    for i=1, partySize do
        local key = "raid" .. i
        if healerData[UnitGUID(key)] then
            if healerData[UnitGUID(key)].ezLoatheb then
                healerRotationList[#healerRotationList + 1] = UnitGUID(key)
            else
                nonHealerList[#nonHealerList + 1] = UnitGUID(key)
            end
        end
    end
end

--NEW
local function removeExpiredHealers()
    local currentTime = GetTime()
    for unitGUID, val in pairs(healerData) do
        if val.lastPingTime and currentTime - val.lastPingTime > 12 then
            val.ezLoatheb = false
        end
    end
end

local function checkPlayerDebuff(playerInfo)
    local i = 1
    local debbuffSlot = 1
    local name, icon, count, debuffType, duration, expirationTime, source, isStealable, nameplateShowPersonal, spellID, canApplyAura, isBossDebuff, castByPlayer, nameplateShowAll, timeMod = UnitDebuff(playerInfo.unitID, i)
    local corruptedMind = false
    while (name) do
        i = i+1
        if corruptedMindID[spellID] then
            corruptedMind = true
        end
        name, icon, count, debuffType, duration, expirationTime, source, isStealable, nameplateShowPersonal, spellID, canApplyAura, isBossDebuff, castByPlayer, nameplateShowAll, timeMod = UnitDebuff(playerInfo.unitID, i)
    end
    if corruptedMind and playerInfo.healingDebuff == 0 then
        playerInfo.healingDebuff = 61
    end
    if not corruptedMind then
        playerInfo.healingDebuff = 0
    end
end

-- Should not be called in combat
-- TODO: remove this
local function updateRegisteredHealerList()
    if InCombatLockdown() then
        needsUpdate = true
        return
    end
    registeredHealers = {}
    for i=1, #unregisteredHealers do
        if UnitIsUnit(unregisteredHealers[i].unitID, "player") or unregisteredHealers[i].ezLoatheb then
            registeredHealers[#registeredHealers + 1] = {
                unitID = unregisteredHealers[i].unitID, 
                unitName = unregisteredHealers[i].unitName, 
                healAmount = unregisteredHealers[i].healAmount or 2300, 
                healingDebuff = 0, 
                active = true
            }
        end
    end
end

-- Only returns true if a previously unregistered healer is now registered
local function updateHealerRegistration(sender, senderHealAmount)
    local _, _, name, realm = string.find(sender, "(.+)-(%a+)")
    if not name then
        return false
    end
    local healerInfo = healerData[UnitGUID(name)]
    if healerInfo then
        healerInfo.lastPingTime = GetTime()
        if not healerInfo.ezLoatheb then
            healerInfo.ezLoatheb = true
            healerInfo.healAmount = senderHealAmount
            if InCombatLockdown() then
                needsUpdate = true
                return false
            end
            return true
        end
    end
    return false
end

-- Display registered healers
local function recreateHealerList()
    updateHealerRotationList()

    -- Display healer rotation
    local j = 1
    for i=1, #healerRotationList do
        local textKey = "text" .. i
        if not loathebMainFrame[textKey] then loathebMainFrame[textKey] = loathebMainFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal"); end
        loathebMainFrame[textKey]:Show()
        loathebMainFrame[textKey]:SetPoint("LEFT", loathebMainFrame, "TOPLEFT", 10, -15*j)
        loathebMainFrame[textKey]:SetTextColor(0, 0.8, 0)
        loathebMainFrame[textKey]:SetText(healerData[healerRotationList[i]].unitName)
        healerData[healerRotationList[i]].textKey = textKey
        j = j + 1
    end

    -- Divider
    local textKey = "text" .. j
    if not loathebMainFrame[textKey] then loathebMainFrame[textKey] = loathebMainFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal"); end
    loathebMainFrame[textKey]:Show()
    loathebMainFrame[textKey]:SetPoint("LEFT", loathebMainFrame, "TOPLEFT", 10, -15*j)
    loathebMainFrame[textKey]:SetTextColor(1, 1, 1)
    loathebMainFrame[textKey]:SetText("---No Addon---")
    j = j + 1

    -- Healers not part of rotation
    for i=1, #nonHealerList do
        local textKey = "text" .. j
        if not loathebMainFrame[textKey] then loathebMainFrame[textKey] = loathebMainFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal"); end
        loathebMainFrame[textKey]:Show()
        loathebMainFrame[textKey]:SetPoint("LEFT", loathebMainFrame, "TOPLEFT", 10, -15*j)
        loathebMainFrame[textKey]:SetTextColor(0, 0.8, 0)
        loathebMainFrame[textKey]:SetText(healerData[nonHealerList[i]].unitName)
        healerData[nonHealerList[i]].textKey = textKey
        j = j + 1
    end

    -- Resize main frame
    loathebMainFrame:SetSize(120, 30 + 15*#healerRotationList + 15*#nonHealerList)

    -- Hide unused text lines
    for k = j, 40 do
        local textKey = "text" .. k
        if loathebMainFrame[textKey] then
            loathebMainFrame[textKey]:Hide()
        else
            return
        end
    end
end

-- Update health and incomming healing values for main tank
local function updateHealthValues()
    local unitHealth = UnitHealth(mainTank)
    local maxHealth = UnitHealthMax(mainTank)
    local healthRatio = unitHealth/maxHealth

    loathebMainFrame.mainTankHealth.healthBar:SetMinMaxValues(0, maxHealth)
    loathebMainFrame.mainTankHealth.healthBar:SetValue(unitHealth)

    --subframe.playerHealBar:SetValue(playerHealAmount + unitHealth)
    --subframe.otherHealBar:SetValue(prePlayerHealAmount + unitHealth)
    --subframe.overhealBar:SetValue(overhealAmount)
end

local function createUIFrames()
    if InCombatLockdown() then
        needsUpdate = true
        return
    end
    loathebMainFrame:SetMovable(true)
    loathebMainFrame:EnableMouse(true)
    loathebMainFrame:RegisterForDrag("LeftButton")
    loathebMainFrame:SetScript("OnDragStart", function()
        loathebMainFrame:StartMoving()
    end)
    -- Healer List Frame
    loathebMainFrame:SetScript("OnDragStop", loathebMainFrame.StopMovingOrSizing)
    if not loathebMainFrame.texture then loathebMainFrame.texture = loathebMainFrame:CreateTexture(nil, "BACKGROUND"); end
    loathebMainFrame.texture:SetAllPoints(loathebMainFrame)
    loathebMainFrame.texture:SetColorTexture(0.3,0.3,0.3)
    loathebMainFrame.texture:SetAlpha(0.5)

    -- Clickable Indicator
    if not loathebMainFrame.indicatorFrame then loathebMainFrame.indicatorFrame = CreateFrame("FRAME", "LoathebIndicatorFrame", loathebMainFrame) end;
    loathebMainFrame.indicatorFrame:SetFrameStrata("BACKGROUND")
    loathebMainFrame.indicatorFrame:SetFrameLevel(100)
    loathebMainFrame.indicatorFrame:SetMovable(true)
    loathebMainFrame.indicatorFrame:EnableMouse(true)
    loathebMainFrame.indicatorFrame:RegisterForDrag("LeftButton")
    loathebMainFrame.indicatorFrame:SetScript("OnDragStart", function()
        loathebMainFrame:StartMoving()
    end)
    loathebMainFrame.indicatorFrame:SetScript("OnDragStop", function() loathebMainFrame:StopMovingOrSizing() end)

    loathebMainFrame.indicatorFrame:SetSize(100, 60)
    loathebMainFrame.indicatorFrame:SetPoint("CENTER", loathebMainFrame, "CENTER", -125, -10)
    if not loathebMainFrame.indicatorFrame.texture then loathebMainFrame.indicatorFrame.texture = loathebMainFrame.indicatorFrame:CreateTexture(nil, "BACKGROUND"); end
    loathebMainFrame.indicatorFrame.texture:SetAllPoints(loathebMainFrame.indicatorFrame)
    loathebMainFrame.indicatorFrame.texture:SetColorTexture(0.3, 0.3, 0.3)
    loathebMainFrame.indicatorFrame.texture:SetAlpha(0.75)
    if not loathebMainFrame.indicatorFrame.text then loathebMainFrame.indicatorFrame.text = loathebMainFrame.indicatorFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal"); end
    loathebMainFrame.indicatorFrame.text:SetPoint("CENTER", loathebMainFrame.indicatorFrame, "CENTER", 0, -15)
    loathebMainFrame.indicatorFrame.text:SetWidth(100)

    -- Main Tank Health Frame
    if not loathebMainFrame.mainTankHealth then loathebMainFrame.mainTankHealth = CreateFrame("FRAME", "LoathebMainTankHealthFrame", loathebMainFrame) end;
    loathebMainFrame.mainTankHealth:SetSize(85, 25)
    loathebMainFrame.mainTankHealth:SetPoint("CENTER", loathebMainFrame, "CENTER", -125, 0)
    if not loathebMainFrame.mainTankHealth.texture then loathebMainFrame.mainTankHealth.texture = loathebMainFrame:CreateTexture(nil, "BACKGROUND"); end
    loathebMainFrame.mainTankHealth.texture:SetAllPoints(loathebMainFrame.mainTankHealth)
    loathebMainFrame.mainTankHealth.texture:SetColorTexture(0, 0, 0)

    if not loathebMainFrame.mainTankHealth.click then loathebMainFrame.mainTankHealth.click = CreateFrame("BUTTON", "LoathebMainTankHealthClick", loathebMainFrame.mainTankHealth, "SecureActionButtonTemplate"); end
    loathebMainFrame.mainTankHealth.click:SetAllPoints(loathebMainFrame.mainTankHealth)
    loathebMainFrame.mainTankHealth.click:RegisterForClicks("AnyDown")

    -- Main Tank Health Border
    if not loathebMainFrame.mainTankHealth.border then loathebMainFrame.mainTankHealth.border = CreateFrame("FRAME", "LoathebMainTankHealthFrame", loathebMainFrame.mainTankHealth) end;
    loathebMainFrame.mainTankHealth.border:SetFrameStrata("BACKGROUND")
    loathebMainFrame.mainTankHealth.border:SetFrameLevel(200)
    loathebMainFrame.mainTankHealth.border:SetSize(90, 30)
    loathebMainFrame.mainTankHealth.border:SetPoint("CENTER", loathebMainFrame, "CENTER", -125, 0)
    if not loathebMainFrame.mainTankHealth.border.texture then loathebMainFrame.mainTankHealth.border.texture = loathebMainFrame.mainTankHealth:CreateTexture(nil, "BACKGROUND"); end
    loathebMainFrame.mainTankHealth.border.texture:SetAllPoints(loathebMainFrame.mainTankHealth.border)
    loathebMainFrame.mainTankHealth.border.texture:SetColorTexture(0.2, 0.2, 0.2)
    --loathebMainFrame.mainTankHealth.border.texture:SetAlpha(0.75)

    -- Main Tank Health Bar
    if not loathebMainFrame.mainTankHealth.healthBar then loathebMainFrame.mainTankHealth.healthBar = CreateFrame("StatusBar", nil, loathebMainFrame.mainTankHealth); end
    loathebMainFrame.mainTankHealth.healthBar:SetFrameStrata("MEDIUM")
    loathebMainFrame.mainTankHealth.healthBar:SetFrameLevel(300)
    loathebMainFrame.mainTankHealth.healthBar:SetSize(85, 25)
    loathebMainFrame.mainTankHealth.healthBar:SetPoint("CENTER", loathebMainFrame.mainTankHealth, "CENTER", 0, 0)

    if not loathebMainFrame.mainTankHealth.healthBar.bgTexture then loathebMainFrame.mainTankHealth.healthBar.bgTexture = loathebMainFrame.mainTankHealth.healthBar:CreateTexture(nil, "BACKGROUND"); end
    loathebMainFrame.mainTankHealth.healthBar.bgTexture:SetAllPoints(loathebMainFrame.mainTankHealth)
    loathebMainFrame.mainTankHealth.healthBar.bgTexture:SetColorTexture(0,0,0)

    if not loathebMainFrame.mainTankHealth.healthBar.texture then loathebMainFrame.mainTankHealth.healthBar.texture = loathebMainFrame.mainTankHealth.healthBar:CreateTexture(nil, "BACKGROUND"); end
    loathebMainFrame.mainTankHealth.healthBar.texture:SetColorTexture(0.7,1,0.7)
    loathebMainFrame.mainTankHealth.healthBar.texture:SetGradient("VERTICAL", 0.45,0.45,0.45, 0.65,0.65,0.65)
    loathebMainFrame.mainTankHealth.healthBar:SetStatusBarTexture(loathebMainFrame.mainTankHealth.healthBar.texture)

    -- Main Tank Text
    if not loathebMainFrame.mainTankHealth.textFrame then loathebMainFrame.mainTankHealth.textFrame = CreateFrame("StatusBar", nil, loathebMainFrame.mainTankHealth); end
    if not loathebMainFrame.mainTankHealth.text then loathebMainFrame.mainTankHealth.text = loathebMainFrame.mainTankHealth.textFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal"); end
    loathebMainFrame.mainTankHealth.text:SetPoint("BOTTOM", loathebMainFrame.mainTankHealth, "TOP", 0, -17)
    loathebMainFrame.mainTankHealth.textFrame:SetFrameStrata("MEDIUM")
    loathebMainFrame.mainTankHealth.textFrame:SetFrameLevel(400)
    if mainTank then
        loathebMainFrame.mainTankHealth.text:SetText(UnitName(mainTank))
        loathebMainFrame.mainTankHealth.healthBar:SetMinMaxValues(0, UnitHealthMax(mainTank))
        loathebMainFrame.mainTankHealth.healthBar:SetValue(UnitHealth(mainTank))
        loathebMainFrame.mainTankHealth.click:SetAttribute("unit", mainTank)
        loathebMainFrame.mainTankHealth.click:SetAttribute("type", "target")
    end

    recreateHealerList()
end

local function updateHealingRecommendation()
    healAmount = 0
    -- Get available healing amount from higher priority healers
    -- Check if player is CD first
    local i = 1
    while healerRotationList[i] and not UnitIsUnit(healerData[healerRotationList[i]].unitID, "player") do 
        if healerData[healerRotationList[i]].healingDebuff == 0 and healerData[healerRotationList[i]].active then
            if healerData[healerRotationList[i]].healAmount then
                healAmount = healAmount + healerData[healerRotationList[i]].healAmount
            end
        end
        i = i + 1
    end
    if not healerRotationList[i] then
        return
    end
    if healerData[healerRotationList[i]].healingDebuff > 0 then
        -- CD
        playerShouldHeal = false
        loathebMainFrame.indicatorFrame.texture:SetColorTexture(0.3, 0.3, 0.3)
        loathebMainFrame.indicatorFrame.text:SetText(healerData[healerRotationList[i]].healingDebuff)
    elseif mainTankHealthDiff + healthBuffer > healAmount + myHealAmount then
        -- GREEN: Cast Heal
        if not playerShouldHeal then
            PlaySoundFile("Interface\\AddOns\\EZLoatheb\\assets\\bell.mp3", "Master")
        end
        playerShouldHeal = true
        loathebMainFrame.indicatorFrame.texture:SetColorTexture(0, 1, 0)
        loathebMainFrame.indicatorFrame.text:SetText("HEAL")
    elseif healAmount < 4000 then
        -- YELLOW: Get Ready To Heal
        playerShouldHeal = false
        loathebMainFrame.indicatorFrame.texture:SetColorTexture(0.8, 0.8, 0.2)
        loathebMainFrame.indicatorFrame.text:SetText("GET READY")
    else
        -- Not next in line to heal
        playerShouldHeal = false
        loathebMainFrame.indicatorFrame.texture:SetColorTexture(0.3, 0.3, 0.3)
        loathebMainFrame.indicatorFrame.text:SetText("WAIT")
    end
end

local function updateHealerDisplay()
    for unitGUID, healerInfo in pairs(healerData) do
        loathebMainFrame[healerInfo.textKey]:SetText(healerInfo.unitName .. " " .. healerInfo.healingDebuff)
        if not healerInfo.active then
            loathebMainFrame[healerInfo.textKey]:SetTextColor(0.5, 0.5, 0.5)
        elseif healerInfo.healingDebuff > 0 then
            loathebMainFrame[healerInfo.textKey]:SetTextColor(0.8, 0, 0)
        else
            loathebMainFrame[healerInfo.textKey]:SetTextColor(0, 0.8, 0)
        end
    end
end

local function healerDebuffTick()
    for unitGUID, healerInfo in pairs(healerData) do
        if healerInfo.healingDebuff > 0 then
            healerInfo.healingDebuff = healerInfo.healingDebuff - 1
        end
    end
end

local function UpdateFrequent(self, timeDelta)
    if checkResetTimer("UPDATE_FREQUENT", timeDelta, 0.2) then
        for unitGUID, healerInfo in pairs(healerData) do
            checkPlayerDebuff(healerInfo)
            if UnitIsDeadOrGhost(healerInfo.unitID) or not UnitIsConnected(healerInfo.unitID) then
                healerInfo.active = false
            else
                healerInfo.active = true
            end
        end
    end
    if checkResetTimer("UPDATE_SEC", timeDelta, 1) then
        healerDebuffTick()
        updateHealerDisplay()
        updateHealingRecommendation()
        if not InCombatLockdown() and needsUpdate then
            needsUpdate = false
            updateHealerList()
            --updateRegisteredHealerList()
            createUIFrames()
        end
    end
    if checkResetTimer("UPDATE_MED", timeDelta, 10) then
        pingAllHealers()
        removeExpiredHealers()
    end
end

local function main()

    function eventHandlers:ADDON_LOADED(addonName)
        if addonName == "EZLoatheb" then
            getMainTank()
            updateHealerList()
            createUIFrames()

            if UnitInRaid("player") == nil then
                loathebHide()
            end
        end
    end

    -- Handler for party changes
    function eventHandlers:GROUP_ROSTER_UPDATE()
        if UnitInRaid("player") ~= nil then
            getMainTank()
            updateHealerList()
            createUIFrames()

            if wasInParty == false then
                wasInParty = true
                loathebShow()
            end
        else
            wasInParty = false
            loathebHide()
        end
    end

    function eventHandlers:CHAT_MSG_ADDON(prefix, message, type, sender)
        if prefix == COMM_PREFIX then
            local messagePrefix, senderHealAmount = message:match("([^,]+),([^,]+)")
            if messagePrefix == "ping" then
                sendAddonMessage("pong," .. myHealAmount, sender);
                local updated = updateHealerRegistration(sender, tonumber(senderHealAmount))
                if updated then
                    --updateRegisteredHealerList()
                    createUIFrames()
                end
            elseif messagePrefix == "pong" then
                local updated = updateHealerRegistration(sender, tonumber(senderHealAmount))
                if updated then
                    --updateRegisteredHealerList()
                    createUIFrames()
                end
            end
        end
    end

    function eventHandlers:UNIT_HEALTH(unitID)
        if mainTank and unitID == mainTank then
            local oldHealthVal = mainTankHealthDiff
            mainTankHealthDiff = UnitHealthMax(mainTank) - UnitHealth(mainTank)
            if oldHealthVal ~= mainTankHealthDiff then
                updateHealingRecommendation()
            end
            
            -- Update health display
            updateHealthValues()
        end
    end

    function eventHandlers:PLAYER_TARGET_CHANGED()
        if mainTank and UnitIsUnit(mainTank, "playertarget") then
            loathebMainFrame.mainTankHealth.border.texture:SetColorTexture(0.9, 0.9, 0.9)
        else
            loathebMainFrame.mainTankHealth.border.texture:SetColorTexture(0.2, 0.2, 0.2)
        end
    end

    -- Attach all handlers to discoMainFrame
    loathebMainFrame:SetScript("OnEvent", function(self, event, ...)
        eventHandlers[event](self, ...); -- call one of the functions above
    end)
    for k, v in pairs(eventHandlers) do
        loathebMainFrame:RegisterEvent(k); -- Register all events for which handlers have been defined
    end
    loathebMainFrame:SetScript("OnUpdate", UpdateFrequent)


end
local classFilename = UnitClassBase("player")
if classFilename == "PALADIN" or classFilename == "PRIEST" or classFilename == "DRUID" or classFilename == "SHAMAN" then
    main()
end